'use strict';

var _require = require('lodash'),
    get = _require.get,
    isArray = _require.isArray,
    isObject = _require.isObject,
    mergeWith = _require.mergeWith,
    omitBy = _require.omitBy,
    set = _require.set,
    sortBy = _require.sortBy;

var _require2 = require('./constants'),
    ROOT_KEY = _require2.ROOT_KEY,
    TSIL_KEY = _require2.TSIL_KEY,
    TSIL_VALUE = _require2.TSIL_VALUE;

/**
 * Customizer for the merge method.
 * If destination is an array and the source is an object
 * return the source without any change.
 *
 * @param  {Any} destination
 * @param  {Any} source
 * @return {Any}
 */


var mergeObjectWithArray = function mergeObjectWithArray(destination, source) {
  var destinationIsArray = isArray(destination);
  var sourceIsObject = !isArray(source) && isObject(source);

  if (destinationIsArray && sourceIsObject) {
    return source;
  }
};

/**
 * Given the node[TSIL_VALUE] return either a children (object)
 * filtered value or the raw one if it's not an object itself
 *
 * @param  {Any} value - node[TSIL_VALUE]
 * @return {Any} withoutChildren - value without children nodes
 */
var withoutChildren = function withoutChildren(value) {
  return isObject(value) ? omitBy(value, isObject) : value;
};

/**
 * Given a sorted list return an object with the
 * original structure.
 * Create an object with the node values applied by the path.
 * Do not include the children because it'll override
 * modified values.
 *
 * @param  {Any[]} sorted   - Tsil flatten list sorted by depth
 * @return {Object} deflat  - Rebuilt object under ROOT_KEY param
 */
var deflatten = function deflatten(sorted) {
  return sorted.reduce(function (memo, node) {
    var path = node[TSIL_KEY].path;
    var patch = set({}, path, withoutChildren(node[TSIL_VALUE]));

    return mergeWith(memo, patch, mergeObjectWithArray);
  }, {});
};

/**
 * Exported deflatten method.
 * Given a tsil flatten list return the rebuilt object.
 * Sort by depth to ensure that deepest children are
 * evaluated first.
 *
 * @param  {Any[]} flat
 * @return {Object}
 */
module.exports = function (flat) {
  var sorted = sortBy(flat, function (x) {
    return -x[TSIL_KEY].depth;
  });
  var deflat = deflatten(sorted);

  return get(deflat, ROOT_KEY);
};